package com.engine.jucailinkq.attendance.workflow.action.askforleave;

import com.engine.common.util.ServiceUtil;
import com.engine.jucailinkq.attendance.enums.AccountingUnitEnum;
import com.engine.jucailinkq.attendance.enums.AttendanceItemTypeEnum;
import com.engine.jucailinkq.attendance.workflow.enums.WorkFlowTypeEnum;
import com.engine.jucailinkq.attendance.workflow.service.AskForLeaveService;
import com.engine.jucailinkq.attendance.workflow.service.impl.AskForLeaveServiceImpl;
import com.engine.jucailinkq.common.util.CommonUtil;
import com.engine.jucailinkq.common.util.DateUtil;
import com.engine.jucailinkq.common.util.DbTools;
import com.engine.jucailinkq.common.util.Utils;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import weaver.general.Util;
import weaver.hrm.User;
import weaver.interfaces.workflow.action.Action;
import weaver.soa.workflow.request.RequestInfo;

import java.math.BigDecimal;
import java.math.MathContext;
import java.math.RoundingMode;
import java.time.ZoneOffset;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Author: sy
 * @Description: 批量请假-提交节点附加操作
 * @Date: 2024/6/28
 **/
@Slf4j
public class BatchAskForLeaveWorkFlowSubmitAction implements Action {
    private AskForLeaveService getAskForLeaveService(User user) {
        return ServiceUtil.getService(AskForLeaveServiceImpl.class,user);
    }
    @Override
    public String execute(RequestInfo requestInfo) {
        String requestid = requestInfo.getRequestid();
        int billid = requestInfo.getRequestManager().getBillid();
//        String requestid = "921921";
//        int billid = 62;
        log.info("billid : [{}]",billid);
        User user = new User(Integer.parseInt("1"));

        try {
            // 流程表单主表数据
            HashMap<String, String> mainTableData = CommonUtil.getMainTableInfo(requestInfo, WorkFlowTypeEnum.ASKFORLEAVE,requestInfo.getWorkflowid());
//            String querysql = "select * from formtable_main_228 where id=?";
//            Map<String,Object> mainTableData = DbTools.getSqlToMap(querysql,billid);
            log.info("mainTableData : [{}]", mainTableData);
            // 流程表单明细表1数据
            List<Map<String, String>> detailTableData = CommonUtil.getDetailTableInfo(requestInfo, 0,WorkFlowTypeEnum.ASKFORLEAVE,requestInfo.getWorkflowid());
            // querysql = "select * from formtable_main_228_dt1 where mainid=?";
            //List<Map<String, String>> detailTableData = DbTools.getSqlToListStr(querysql,billid);
            log.info("detailTableData : [{}]", detailTableData);
            String detail1TableName = CommonUtil.getDetailTableInfoTableName(requestInfo.getDetailTableInfo().getDetailTable()[0].getTableDBName(),0,WorkFlowTypeEnum.ASKFORLEAVE,requestInfo.getWorkflowid());
            //String detail1TableName = "formtable_main_228_dt1";
            log.info("detail1TableName : [{}]", detail1TableName);
            String detail2TableName = CommonUtil.getDetailTableInfoTableName(requestInfo.getDetailTableInfo().getDetailTable()[1].getTableDBName(),1,WorkFlowTypeEnum.ASKFORLEAVE,requestInfo.getWorkflowid());
            //String detail2TableName= "formtable_main_228_dt2";
            log.info("detail2TableName : [{}]", detail2TableName);
            //流程表单明细表3数据
            List<Map<String, String>> detailTable3Data = CommonUtil.getDetailTableInfo(requestInfo,2, WorkFlowTypeEnum.ASKFORLEAVE,requestInfo.getWorkflowid());
            //querysql = "select * from formtable_main_228_dt3 where mainid=?";
            //List<Map<String, String>> detailTable3Data = DbTools.getSqlToListStr(querysql,billid);
            log.info("detailTable3Data : [{}]", detailTable3Data);
            //20241127需求变更，明细表1存在数据时，直接走原有校验逻辑；明细表1没有数据时，就根据明细表3重新生成明细表1数据，然后继续原有校验逻辑
            Map<String, Object> params = new HashMap<>();
            List<Map<String, String>> reCreateDetailList = new ArrayList<>();
            if (detailTable3Data.size() > 0) {
                List<Map<String, Object>> insertDetail1List = new ArrayList<>();
                Map<String, Object> itemResult = new HashMap<>();
                List<Map<String, String>> itemDetailList = new ArrayList<>();
                for (Map<String, String> detailTable3Item : detailTable3Data) {
                    detailTable3Item.put("jqlx", detailTable3Item.get("qjlx"));
                    detailTable3Item.put("cxjqj", detailTable3Item.get("qjfs"));

                    params.put("mainTableData", detailTable3Item);
                    params.put("detailTableData", reCreateDetailList);
                    itemResult = getAskForLeaveService(user).generateLeaveInfoList(params);
                    log.info("itemResult : [{}]", itemResult);
                    if ("false".equals(Util.null2String(itemResult.get("status"))) || itemResult.get("errorInfo") != null) {
                        String message = "生成请假明细失败：" + Util.null2String(itemResult.get("errorInfo"));
                        log.error(message);
                        requestInfo.getRequestManager().setMessageid("11111" + requestid + "22222");
                        requestInfo.getRequestManager().setMessagecontent(message);
                        return Action.FAILURE_AND_CONTINUE;
                    } else if (itemResult.get("data") != null) {
                        itemDetailList = (List<Map<String, String>>) itemResult.get("data");
                        if (itemDetailList.size() > 0) {
                            reCreateDetailList.addAll(itemDetailList);
                            Map<String, Object> insertItem;
                            for (Map<String, String> item : itemDetailList) {
                                insertItem = new HashMap<>();
                                insertItem.put("mainid", mainTableData.get("id"));
                                insertItem.put("ksrq", item.getOrDefault("ksrq", ""));
                                insertItem.put("kssj", item.getOrDefault("kssj", ""));
                                insertItem.put("jsrq", item.getOrDefault("jsrq", ""));
                                insertItem.put("jssj", item.getOrDefault("jssj", ""));
                                insertItem.put("qjsc", item.getOrDefault("qjsc", ""));
                                insertItem.put("qtj", item.getOrDefault("qtj", "0"));
                                insertItem.put("btj", item.getOrDefault("btj", "0"));
                                insertItem.put("cxqj", "0");
                                insertItem.put("qjr", item.getOrDefault("qjr", ""));
                                insertItem.put("qjlx", item.getOrDefault("qjlx", ""));
                                insertItem.put("jqye", item.getOrDefault("jqye", ""));
                                insertDetail1List.add(insertItem);
                            }

                        }
                    }
                }
                log.info("insertDetail1List : [{}]", insertDetail1List);
                String deleteSql="delete from "+detail1TableName+" where mainid=?";
                if (insertDetail1List.size() > 0 && DbTools.update(deleteSql,mainTableData.get("id")) && !CommonUtil.insertBatch(insertDetail1List, detail1TableName)) {
                        //更新流程明细表2失败
                    String message = "新增流程明细表1失败";
                    log.error(message);
                    requestInfo.getRequestManager().setMessageid("11111" + requestid + "22222");
                    requestInfo.getRequestManager().setMessagecontent(message);
                    return Action.FAILURE_AND_CONTINUE;
                }
                String sql = "select * from " + detail1TableName + " where mainid = " + mainTableData.get("id");
                List<Map<String, Object>> data = DbTools.getSqlToList(sql);
                if (data.size() > 0) {
                    detailTableData = data.stream()
                            .map(map -> map.entrySet()
                                    .stream()
                                    .collect(Collectors.toMap(Map.Entry::getKey, e -> Util.null2String(e.getValue()))))
                            .collect(Collectors.toList());;
                }
                log.info("detailTableData : [{}]", detailTableData);
            }
            if (detailTableData.size() == 0) {
                String message = "无法生成有效请假明细时长，请检查请假基础选择信息！";
                log.error(message);
                requestInfo.getRequestManager().setMessageid("11111" + requestid + "22222");
                requestInfo.getRequestManager().setMessagecontent(message);
                return Action.FAILURE_AND_CONTINUE;
            }
            //第一笔开始时间
            List<String> startDateList = detailTableData.stream().map(map -> map.get("ksrq")).collect(Collectors.toList());
            String minStartDate = startDateList.stream().min(Comparator.comparing(e-> DateUtil.getTime(e).toInstant(ZoneOffset.of("+8")).toEpochMilli())).get();
            //请假人员列表
            List<String> leaveEmpIdList = detailTableData.stream().map(map -> map.get("qjr")).distinct().collect(Collectors.toList());

            //请假人姓名映射
            String sql;
            Map<String, String> empIdToName = new HashMap<>();
            if (leaveEmpIdList.size() > 0) {
                sql = "select id, lastname from hrmresource where id in (" + String.join(",",leaveEmpIdList) + ")";
                List<Map<String, Object>> data = DbTools.getSqlToList(sql);
                empIdToName = data.stream().collect(Collectors.toMap(e->Util.null2String(e.get("id")),e->Util.null2String(e.get("lastname"))));
            }
            //获取假期类型的考勤项目
            sql = "select id, mc, hsdw, zdyzsl, yxyz,qzsyyxjb,jcbyxsyqjb,yxsydjb from uf_jcl_kq_kqxm where xmlx=?";
            List<Map<String,Object>> holidayItemList = DbTools.getSqlToList(sql, AttendanceItemTypeEnum.HOLIDAY.getKey());
            Map<String, Map<String,Object>> kqxmInfo = holidayItemList.stream().collect(Collectors.toMap(e->Util.null2String(e.get("id")), e->e));

            List<Map<String, Object>> detailTable2 = Lists.newArrayList();
            Map<String, Object> holidayBalanceMap = new HashMap<>();
            //需要校验假期额度的假期类型
            List<String> checkAmountJqIdList = CommonUtil.getJqInfoWithAmount();
            //根据明细表1，根据请假类型分组，对于需要校验余额的遍历生成明细表2
            Map<String, List<Map<String, String>>> detail1TabDataGroupByJqlx = detailTableData.stream().collect(Collectors.groupingBy(e -> Util.null2String(e.get("qjlx"))));
            //收集假期类型和使用人员id的映射
            Map<String, List<String>> jqlxToEmpIds = new HashMap<>();
            //收集假期类型和使用人员id的映射
            Map<String, List<Map<String, Object>>> jqlxToJqye = new HashMap<>();
            //收集假期余额id和本次流程中使用的小时时长映射
            Map<String, Double> jqyeIdToUseHours = new HashMap<>();
            for(Map.Entry<String,List<Map<String,String>>> entry : detail1TabDataGroupByJqlx.entrySet()) {
                if (checkAmountJqIdList.contains(entry.getKey())) {


                    sql = "select id,mc,hsdw,hsl,jcbyxsyqjb,yxsydjb,qzsyyxjb  from uf_jcl_kq_kqxm  where id=?";
                    Map<String, Object> holidayItem = DbTools.getSqlToMap(sql, entry.getKey());
                    //假期核算单位
                    String hsdw = Util.null2String(holidayItem.get("hsdw"));
                    //遍历生成明细表2
                    List<String> empIds = new ArrayList<>();
                    for (Map<String, String> detailData : entry.getValue()) {
                        String jqyeInfoStr = Util.null2String(detailData.get("jqye"));
                        List<String> jqyeInfoList = Arrays.asList(jqyeInfoStr.split(","));
                        for (String jqyeInfo : jqyeInfoList) {
                            Map<String, Object> detail2Map = Maps.newHashMap();
                            detail2Map.put("glmxid", detailData.get("id"));
                            detail2Map.put("jqye", jqyeInfo.split("_")[0]);
                            detail2Map.put("mainid", mainTableData.get("id"));
                            detail2Map.put("glrq", detailData.get("ksrq"));
//                            if (hsdw.equals(AccountingUnitEnum.DAY.getKey())) {
//                                detail2Map.put("sysc", Utils.divide(Double.parseDouble(jqyeInfo.split("_")[1]), 8));
//                            } else {
//                                detail2Map.put("sysc", jqyeInfo.split("_")[1]);
//                            }
                            //20241118需求变更，时长不进行核算单位倍率转换，按照原本核算单位进行收集和处理
                            detail2Map.put("sysc", jqyeInfo.split("_")[1]);
                            detailTable2.add(detail2Map);
                            //累计假期余额id对应的使用小时数/天数，具体单位根据假别的核算单位
                            jqyeIdToUseHours.put(jqyeInfo.split("_")[0],
                                    jqyeIdToUseHours.getOrDefault(jqyeInfo.split("_")[0], (double) 0) + Double.parseDouble(jqyeInfo.split("_")[1]));
                        }
                        //收集使用该假期类型的人员id
                        empIds.add(Util.null2String(detailData.get("qjr")));
                    }
                    List<Map<String, Object>> holidayPriorityItemList = Lists.newArrayList();
                    CommonUtil.getHolidayPriorityItem(holidayPriorityItemList,kqxmInfo,holidayItem);
                    jqlxToEmpIds.put(entry.getKey(), empIds);
                    for (Map<String, Object> map:holidayPriorityItemList){
                        jqlxToEmpIds.put(map.get("id").toString(),empIds);
                    }

                    holidayPriorityItemList.add(holidayItem);
                    for (Map<String, Object> holidayPriorityItem:holidayPriorityItemList){
                        sql = "select id,jqid,sxrq,ktsc,yxsc,wxsc,yqsxrq,ztsc,ygid from uf_jcl_kq_jqye where jqid=? and sxrq <= ? order by yqsxrq, modedatacreatedate, modedatacreatetime";
                        List<Map<String, Object>> holidayBalanceList = DbTools.getSqlToList(sql, holidayPriorityItem.get("id"), minStartDate);
                        if (holidayBalanceList.size() > 0) {
                            holidayBalanceMap.putAll(holidayBalanceList.stream().collect(Collectors.toMap(e -> e.get("id").toString(), e -> e)));
                            jqlxToJqye.put(holidayPriorityItem.get("id").toString(), holidayBalanceList);
                        }
                    }
                }
            }
            //校验假期余额使用是否满足要求，并且区分核算单位“天”和“小时”
            StringBuilder jqyeCheckFailMessage = new StringBuilder();
            Map<String, String> jqyeIdToJqMc = new HashMap<>();
            Map<String, String> jqyeIdToEmpName = new HashMap<>();
            //获取本次使用的假期余额对应还可使用的假期余额小时数
            List<String> errorMessage = new ArrayList<>();
            Map<String, Double> jqyeIdToCanUseHours = getJqyeCanUseHours(jqlxToEmpIds, jqlxToJqye, errorMessage, empIdToName, kqxmInfo, jqyeIdToJqMc, jqyeIdToEmpName);
            for (Map.Entry<String, Double> entry : jqyeIdToUseHours.entrySet()) {
                if (jqyeIdToCanUseHours.getOrDefault(entry.getKey(), (double) 0) < entry.getValue()) {
                    //记录假期余额不足信息
                    jqyeCheckFailMessage.append(jqyeIdToEmpName.get(entry.getKey())).append("-").append(jqyeIdToJqMc.get(entry.getKey())).append("的假期余额不足！");
                }
            }
            if (errorMessage.size() > 0) {
                jqyeCheckFailMessage.append(errorMessage);
            }
            if (!"".equals(jqyeCheckFailMessage.toString())) {
                log.error(jqyeCheckFailMessage.toString());
                requestInfo.getRequestManager().setMessageid("11111" + requestid + "22222");
                requestInfo.getRequestManager().setMessagecontent(jqyeCheckFailMessage.toString());
                return Action.FAILURE_AND_CONTINUE;
            }
            //明细表2有数据时，更新明细表2入库，并且更新假期余额表
            //更新流程明细表2
            String delteSql = "delete from " + detail2TableName + " where mainid=?";
            DbTools.update(delteSql, billid);
            if (detailTable2.size() > 0 && !CommonUtil.insertBatch(detailTable2, detail2TableName)) {
                //更新流程明细表2失败
                String message = "更新流程明细表2失败";
                log.error(message);
                requestInfo.getRequestManager().setMessageid("11111" + requestid + "22222");
                requestInfo.getRequestManager().setMessagecontent(message);
                return Action.FAILURE_AND_CONTINUE;
            }
            //更新假期余额在途时长
            Map<String, Object> balanceMap = Maps.newHashMap();
            for (Map<String, Object> data : detailTable2) {
                String jqye = Util.null2String(data.get("jqye"));
                double sysc = Double.valueOf(data.get("sysc").toString());
                double havedSysc = balanceMap.get(jqye) == null ? 0 : Double.valueOf(balanceMap.get(jqye).toString());
                sysc = new BigDecimal(String.valueOf(sysc)).add(new BigDecimal(String.valueOf(havedSysc)),new MathContext(5, RoundingMode.HALF_UP)).doubleValue();
                balanceMap.put(jqye, sysc);
            }
            log.info("balanceMap : [{}]",balanceMap);
            for (Map.Entry<String, Object> entry : balanceMap.entrySet()) {
                String id = entry.getKey();
                double ztsc = Double.valueOf(entry.getValue().toString());
                Map<String, Object> map = (Map<String, Object>) holidayBalanceMap.get(id);

                double havedztsc = Util.null2String(map.get("ztsc")).equals("") ? 0 : Double.valueOf(map.get("ztsc").toString());
                double havedwxsc = Util.null2String(map.get("wxsc")).equals("") ? 0 : Double.valueOf(map.get("wxsc").toString());

                double updatedztsc = new BigDecimal(String.valueOf(havedztsc)).add(new BigDecimal(String.valueOf(ztsc)),new MathContext(5, RoundingMode.HALF_UP)).doubleValue();
                double updatedwxsc = new BigDecimal(String.valueOf(havedwxsc)).subtract(new BigDecimal(String.valueOf(ztsc)),new MathContext(5, RoundingMode.HALF_UP)).doubleValue();

                String updateSql = "update uf_jcl_kq_jqye set wxsc=?,ztsc=? where id=?";
                if (!DbTools.update(updateSql, updatedwxsc, updatedztsc, id)) {
                    //更新假期余额失败
                    String message = "更新假期余额失败";
                    log.error(message);
                    requestInfo.getRequestManager().setMessageid("11111" + requestid + "22222");
                    requestInfo.getRequestManager().setMessagecontent(message);
                    return Action.FAILURE_AND_CONTINUE;
                }
            }

        }catch (Exception e){
            log.error("BatchAskForLeaveWorkFlowSubmitAction error : [{}]",e);
            return Action.FAILURE_AND_CONTINUE;
        }

        return Action.SUCCESS;
    }

    private Map<String, Double> getJqyeCanUseHours(Map<String, List<String>> jqlxToEmpIds, Map<String, List<Map<String, Object>>> jqlxToJqye,
                                                   List<String> errorMessage, Map<String, String> empIdToName, Map<String, Map<String, Object>> kqxmInfo,
                                                   Map<String, String> jqyeIdToJqMc, Map<String, String> jqyeIdToEmpName) {
        Map<String, Double> result = new HashMap<>();
        if (jqlxToEmpIds.size() == 0) {
            return result;
        }
        //获取modeId，假期额度规则的modeId
        Map<String,String> formmodeIdMap = Utils.getFormmodeIdMap();
        String modeId = formmodeIdMap.get("uf_jcl_kq_jqed");
        List<Map<String, Object>> jqyeInfoList;
        Map<String,Object> holidayItem = new HashMap<>();
        for (Map.Entry<String, List<String>> entry : jqlxToEmpIds.entrySet()) {
            holidayItem = kqxmInfo.get(entry.getKey());
            jqyeInfoList = jqlxToJqye.get(entry.getKey());
            if (jqyeInfoList == null || jqyeInfoList.size() == 0) {
                continue;
            }
            jqyeInfoList = jqyeInfoList.stream().filter(f -> entry.getValue().contains(f.get("ygid").toString())).collect(Collectors.toList());
            if (jqyeInfoList.size() == 0) {
                continue;
            }
            //获取假期额度规则中额度可休次数、单次最小休时长、额度单位
            Map<String, Object> jqedInfo = new HashMap<>();
            String sql = "select id, eddw, dczskxsc, edbxdcxw from uf_jcl_kq_jqed where jb = ? and gzzt = 0";
            List<Map<String, Object>> jqedData = DbTools.getSqlToList(sql, entry.getKey());
            //按人员分组
            Map<String, List<Map<String, Object>>> empIdToJqyeInfo = jqyeInfoList.stream().collect(Collectors.groupingBy(map -> (String) map.get("ygid")));
            for (Map.Entry<String, List<Map<String, Object>>> entry2 : empIdToJqyeInfo.entrySet()) {
                List<Map<String, Object>> waitDealHolidayBalanceList = entry2.getValue();
                //获取延期失效日期最晚的一条
                Map<String, Object> maxYqsxrqMap = waitDealHolidayBalanceList.stream().reduce((m1, m2) -> m2).orElse(null);

                Integer empPriority = null;
                boolean samePriority = false;
                for (Map<String, Object> jqedItem : jqedData) {
                    String dataId = jqedItem.get("id").toString();
                    //获取假期额度规则适用的人员id列表
                    Map<String, Integer> empInfo = CommonUtil.getEmpIdWithPriority(dataId, modeId);
                    if (empInfo.get(entry2.getKey()) != null) {
                        if (empPriority == null) {
                            jqedInfo = jqedItem;
                            empPriority = empInfo.get(entry2.getKey());
                        } else if (empPriority > empInfo.get(entry2.getKey())) {
                            jqedInfo = jqedItem;
                            empPriority = empInfo.get(entry2.getKey());
                        } else if (empPriority.equals(empInfo.get(entry2.getKey()))) {
                            samePriority = true;
                        }

                    }
                }
                if (empPriority != null && samePriority) {
                    String message = empIdToName.get(entry2.getKey());
                    message = message + "可使用的假别—‘" + holidayItem.get("mc") + "’同一优先级情况下存在超过一种适用的假期额度规则，无法确定唯一规则！";
                    errorMessage.add(message);
                    continue;
                }
                //假期的核算单位，0-天、1-小时
                String hsdw = Util.null2String(holidayItem.get("hsdw"));
                //假期额度的额度单位，0-天、1-小时
                String eddw = Util.null2String(jqedInfo.get("eddw"));
                //额度可修次数
                String allowLeaveNumStr = Util.null2String(jqedInfo.get("edbxdcxw"));
                if (!"".equals(allowLeaveNumStr) && waitDealHolidayBalanceList.size() > 0) {
                    //查询请假记录中，人员id+假期类别id情况下的结果，遍历每条主表数据下的明细数据中使用了哪些假期余额id
                    sql = "select a.id,dt2.jqye from uf_jcl_kq_qjjl a left join uf_jcl_kq_qjjl_dt1 dt1 on dt1.mainid = a.id left join uf_jcl_kq_qjjl_dt2 dt2 on dt2.glmxid = dt1.glmxid " +
                            "where a.jqlx = " + entry.getKey() + " and qjry = " + entry2.getKey() + " and a.cxqj = 0 and dt1.cxqj = 0 and a.jlzt in (0, 1) and dt1.glmxid is not null";
                    List<Map<String, Object>> leaveList = DbTools.getSqlToList(sql);
                    Map<String, Integer> jqyeUseNumInfo = new HashMap<>();
                    List<String> leaveUseList = new ArrayList<>();
                    Integer useNum;
                    for (Map<String, Object> leaveItem : leaveList) {
                        String jqyeId = Util.null2String(leaveItem.get("jqye"));
                        if (!"".equals(jqyeId)) {
                            String leaveInfo = leaveItem.get("id").toString() + "_" + jqyeId;
                            if (leaveUseList.size() == 0) {
                                leaveUseList.add(leaveInfo);
                                jqyeUseNumInfo.put(jqyeId, 1);
                            } else if (!leaveUseList.contains(leaveInfo)) {
                                leaveUseList.add(leaveInfo);
                                useNum = jqyeUseNumInfo.get(jqyeId);
                                jqyeUseNumInfo.put(jqyeId, useNum == null ? 1 : ++useNum);
                            }
                        }
                    }
                    //筛选次数未使用完的假期余额
                    waitDealHolidayBalanceList = waitDealHolidayBalanceList.stream().filter(f -> {
                        int beforeUseNum = jqyeUseNumInfo.get(f.get("id").toString()) == null ? 0 : jqyeUseNumInfo.get(f.get("id").toString());
                        return Integer.parseInt(allowLeaveNumStr) > beforeUseNum;
                    }).collect(Collectors.toList());
                }
                //判断是否可预支
                String allowAdvance = Util.null2String(holidayItem.get("yxyz"));
                String advanceSc = Util.null2String(holidayItem.get("zdyzsl"));
                //遍历可使用的假期余额
                for (Map<String, Object> holidayBalance : waitDealHolidayBalanceList) {
                    String jqyeId = Util.null2String(holidayBalance.get("id"));
                    double wxsc = Utils.convertDouble(holidayBalance.get("wxsc"));
                    //如果该条假期余额可以预支，则在原有未休时长基础上增加预支时长
                    if ("1".equals(allowAdvance) && maxYqsxrqMap != null && Util.null2String(maxYqsxrqMap.get("id")).equals(jqyeId)) {
                        wxsc = wxsc + Utils.convertDouble(advanceSc);
                    }
                    result.put(jqyeId, wxsc);
                    jqyeIdToJqMc.put(jqyeId, holidayItem.get("mc").toString());
                    jqyeIdToEmpName.put(jqyeId, empIdToName.get(entry2.getKey()));
                }
            }
        }
        return result;
    }
}
